#ifndef DRAGABLETABLE_H
#define DRAGABLETABLE_H

#include <QWidget>
#include <QStandardItemModel>
#include <QTableView>
#include <QStyledItemDelegate>
#include <QUuid>
#include "../BaseItemDelegate/baseitemdelegate.h"

class DragableTableDelegate : public BaseItemDelegate
{
    Q_OBJECT
public:
    explicit DragableTableDelegate(QObject *parent = nullptr);
    void SetDragTarget(int targetRow, bool isUpon);
public:
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
protected:
    int m_nTargetRow;
    bool m_bIsUpon;
    const int km_nRadius = 8;
};


class DragableTable : public QTableView
{
    Q_OBJECT
public:
    explicit DragableTable(QWidget *parent = nullptr);
    void SetSignalOnly(bool value){m_bSignalOnly = value;}
    void SetDragEnable(bool value);
protected:
    virtual void mouseMoveEvent(QMouseEvent *event) override;
    virtual void mousePressEvent(QMouseEvent *event) override;
    virtual void mouseReleaseEvent(QMouseEvent *event) override;
    virtual void dragMoveEvent(QDragMoveEvent *event) override;
    virtual void dropEvent(QDropEvent *event) override;
    virtual void dragEnterEvent(QDragEnterEvent *event) override;
signals:
    void sig_Drop(int sourceRow, int targetRow);    //信号发生在真正修改table行顺序之前
private:
    QModelIndex m_indexPressed;
    QUuid m_UUID;                       //该table对象唯一标识符(不允许从一个table拖拽到另一个)
    bool m_bSignalOnly;                 //仅发出信号, 而不真正修改table行顺序
    bool m_bDragEnable;                 //是否允许拖拽
    int m_nTargetRow;
    bool m_bIsUpon;
private:
    void SetDragTargetRow(int row, bool isUpon = true);
};

#endif // DRAGABLETABLE_H
