#include "onesettingitem.h"
#include "ui_onesettingitem.h"

#include <QDebug>
#include <QFile>
#include "colordialogapi.h"

OneSettingItem::OneSettingItem(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::OneSettingItem)
{
    ui->setupUi(this);

    m_logger = spdlog::get("EyeMap");
    if(m_logger == nullptr)
    {
        qDebug() << "获取 EyeMap logger 失败";
        return;
    }
    /* 加载QSS */
    QFile fileQss(":/qss/SettingNum/OneItem/OneSettingItem.qss");
    if(fileQss.open(QFile::ReadOnly))
    {
        QString qss = fileQss.readAll();
        this->setStyleSheet(qss);
        fileQss.close();
    } else
    {
        SPDLOG_LOGGER_ERROR(m_logger, "加载QSS文件失败");
    }

    /* 链接信号和槽 */
    connect(ui->pBtn_background, &QPushButton::clicked, this, &OneSettingItem::do_pBtn_background);
}

OneSettingItem::~OneSettingItem()
{
    delete ui;
}


/* 设置序号 */
void OneSettingItem::setNum(int num)
{
    m_num = num;
    ui->label_num->setText(QString::number(num));
}

/* 设置颜色 */
void OneSettingItem::setColor(const QColor& color)
{
    QString qss = QString("background-color: %1").arg(color.name());
    ui->pBtn_background->setStyleSheet(qss);
}

/* 获取颜色 */
void OneSettingItem::do_pBtn_background()
{
    SPDLOG_LOGGER_DEBUG(m_logger, "序号{}点击了背景颜色按钮", m_num);
    QColor color = getColor(nullptr, ColorDlgSkin::DARK);
    if(color.isValid())
    {
        QString qss = QString("background-color: %1").arg(color.name());
        ui->pBtn_background->setStyleSheet(qss);
    }
}

